@@ -1,4 +1,5 @@
 use super::*;
+use crate::error::Result;
 
 #[test]
 fn test_priority_get_from() -> Result<()> {
@@ -6,10 +7,7 @@ fn test_priority_get_from() -> Result<()> {
     let mut p = PriorityAttr::default();
     let result = p.get_from(&m);
     if let Err(err) = result {
-        assert!(
-            stun::error::Error::ErrAttributeNotFound.equal(&err),
-            "unexpected error"
-        );
+        assert_eq!(stun::Error::ErrAttributeNotFound, err, "unexpected error");
     } else {
         panic!("expected error, but got ok");
     }
